@@ -1,11 +1,21 @@
 
+import psutil
+import os
+import numpy as np
 from typing import List, Optional
 from parse import parse
+from platform import system
 from pydantic import BaseModel, ValidationError
+from unsync import unsync
+from multiprocessing import cpu_count
 
 from ..helpers import tab_to_comma
 
 
+CPU_COUNT = cpu_count()
+OS = system()
+
+
 class BaseDataStructure:
     """
     Base data structure class
@@ -80,7 +90,9 @@ class BaseDataStructure:
         return cls.meta_preprocessor(parsed.named)
 
     @classmethod
-    def extract_table(cls, data: str, meta: Optional[dict] = None) -> dict:
+    def extract_table(
+        cls, data: str, meta: Optional[dict] = None, parallel: bool = True,
+    ) -> dict:
         """
         Extract the table rows from the data string.
 
@@ -99,12 +111,12 @@ class BaseDataStructure:
         # Attempt to validate table rows using the row_model. This will attempt to cast
         # any fields that don't match the corresponding row_model field.
         try:
-            table_rows = [cls.row_model(**row).dict() for row in table_rows]
+            table_rows = validate_table_rows(cls.row_model, table_rows, parallel=parallel)
         except ValidationError:
             return None  # return None if unable to validate table rows
 
         # Run the table rows through row_preprocessor:
-        table_rows = [cls.row_preprocessor(row) for row in table_rows]
+        table_rows = preprocess_table_rows(cls.row_preprocessor, table_rows, parallel)
 
         # Run the table rows through table_preprocessor and return:
         return cls.table_preprocessor(table_rows, meta)
@@ -117,11 +129,13 @@ class BaseDataStructure:
         return cls.meta_model(**meta).dict(exclude_unset=True)
 
     @classmethod
-    def validate_table(cls, table_rows):
+    def validate_table(cls, table_rows, parallel=True):
         """
         Validate the table rows using row_model.
         """
-        return [cls.row_model(**row).dict(exclude_unset=True) for row in table_rows]
+        return validate_table_rows(
+            cls.row_model, table_rows, exclude_unset=True, parallel=parallel,
+        )
 
     @classmethod
     @property
@@ -138,3 +152,47 @@ class BaseDataStructure:
         Data structure ID.
         """
         return cls.__name__.split("_")[-1]
+
+
+def unpack_results(tasks):
+    return [rr for tt in tasks for rr in tt.result()]
+
+
+def limit_cpu():
+    p = psutil.Process(os.getpid())
+    if OS == "Windows":
+        p.nice(psutil.BELOW_NORMAL_PRIORITY_CLASS)
+    else:
+        p.nice(19)
+
+
+@unsync(cpu_bound=True)
+def call_row_model(row_model, rows, exclude_unset=False):
+    limit_cpu()
+    return [row_model(**row).dict(exclude_unset=exclude_unset) for row in rows]
+
+
+@unsync(cpu_bound=True)
+def call_row_preprocessor(row_preprocessor, rows):
+    limit_cpu()
+    return [row_preprocessor(row) for row in rows]
+
+
+def validate_table_rows(row_model, table_rows, exclude_unset=False, parallel=True):
+    if parallel:
+        tasks = [
+            call_row_model(row_model, rows.tolist(), exclude_unset)
+            for rows in np.array_split(table_rows, CPU_COUNT)
+        ]
+        return unpack_results(tasks)
+    return [row_model(**row).dict(exclude_unset=exclude_unset) for row in table_rows]
+
+
+def preprocess_table_rows(row_preprocessor, table_rows, parallel=True):
+    if parallel:
+        tasks = [
+            call_row_preprocessor(row_preprocessor, rows.tolist())
+            for rows in np.array_split(table_rows, CPU_COUNT)
+        ]
+        return unpack_results(tasks)
+    return [row_preprocessor(row) for row in table_rows]
